"""
Integration with the Kodexa Platform for processing, storing, and sharing of documents and data.
"""
from .kodexa import RemoteSession, RemotePipeline, RemoteStep, KodexaPlatform
from .client import KodexaClient
